# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .place import (
    PlaceResource,
    AsyncPlaceResource,
    PlaceResourceWithRawResponse,
    AsyncPlaceResourceWithRawResponse,
    PlaceResourceWithStreamingResponse,
    AsyncPlaceResourceWithStreamingResponse,
)
from ...types import multigeocode_search_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.multigeocode_search_response import MultigeocodeSearchResponse

__all__ = ["MultigeocodeResource", "AsyncMultigeocodeResource"]


class MultigeocodeResource(SyncAPIResource):
    @cached_property
    def place(self) -> PlaceResource:
        return PlaceResource(self._client)

    @cached_property
    def with_raw_response(self) -> MultigeocodeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return MultigeocodeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MultigeocodeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return MultigeocodeResourceWithStreamingResponse(self)

    def search(
        self,
        *,
        key: str,
        at: multigeocode_search_params.At,
        query: str,
        city: str | NotGiven = NOT_GIVEN,
        country: str | NotGiven = NOT_GIVEN,
        district: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        radius: str | NotGiven = NOT_GIVEN,
        state: str | NotGiven = NOT_GIVEN,
        street: str | NotGiven = NOT_GIVEN,
        sub_district: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MultigeocodeSearchResponse:
        """
        The method enables searching for known places from multiple data sources

        Use this method to find known places in default or your own custom (proprietary)
        dataset and get a combined search result. It accepts free-form, partially
        correct or even incomplete search texts. Results would be ranked based on the
        search score of a place.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          at: Specify the center of the search context expressed as coordinates.

          query: A free-form, complete or incomplete string to be searched. It allows searching
              for places using keywords or names.

          city: Specifies the primary city of the place.

          country: Country of the search context provided as comma-separated
              [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
              codes.
              Note: Country codes should be provided in uppercase.

          district: Specifies the district of the search place.

          limit: Sets the maximum number of results to be returned.

          radius: Filters the results to places within the specified radius from the 'at'
              location.

              Note: Supports 'meter' (m) and 'kilometer' (km) units. If no radius is given,
              the search method returns as many results as specified in `limit`.

          state: Specifies the state of the search place.

          street: Specifies the street name of the search place.

          sub_district: Specifies the subDistrict of the search place.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/multigeocode/search",
            body=maybe_transform(
                {
                    "at": at,
                    "query": query,
                    "city": city,
                    "country": country,
                    "district": district,
                    "limit": limit,
                    "radius": radius,
                    "state": state,
                    "street": street,
                    "sub_district": sub_district,
                },
                multigeocode_search_params.MultigeocodeSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, multigeocode_search_params.MultigeocodeSearchParams),
            ),
            cast_to=MultigeocodeSearchResponse,
        )


class AsyncMultigeocodeResource(AsyncAPIResource):
    @cached_property
    def place(self) -> AsyncPlaceResource:
        return AsyncPlaceResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMultigeocodeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMultigeocodeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMultigeocodeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncMultigeocodeResourceWithStreamingResponse(self)

    async def search(
        self,
        *,
        key: str,
        at: multigeocode_search_params.At,
        query: str,
        city: str | NotGiven = NOT_GIVEN,
        country: str | NotGiven = NOT_GIVEN,
        district: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        radius: str | NotGiven = NOT_GIVEN,
        state: str | NotGiven = NOT_GIVEN,
        street: str | NotGiven = NOT_GIVEN,
        sub_district: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MultigeocodeSearchResponse:
        """
        The method enables searching for known places from multiple data sources

        Use this method to find known places in default or your own custom (proprietary)
        dataset and get a combined search result. It accepts free-form, partially
        correct or even incomplete search texts. Results would be ranked based on the
        search score of a place.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          at: Specify the center of the search context expressed as coordinates.

          query: A free-form, complete or incomplete string to be searched. It allows searching
              for places using keywords or names.

          city: Specifies the primary city of the place.

          country: Country of the search context provided as comma-separated
              [ISO 3166-1 alpha-3](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-3) country
              codes.
              Note: Country codes should be provided in uppercase.

          district: Specifies the district of the search place.

          limit: Sets the maximum number of results to be returned.

          radius: Filters the results to places within the specified radius from the 'at'
              location.

              Note: Supports 'meter' (m) and 'kilometer' (km) units. If no radius is given,
              the search method returns as many results as specified in `limit`.

          state: Specifies the state of the search place.

          street: Specifies the street name of the search place.

          sub_district: Specifies the subDistrict of the search place.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/multigeocode/search",
            body=await async_maybe_transform(
                {
                    "at": at,
                    "query": query,
                    "city": city,
                    "country": country,
                    "district": district,
                    "limit": limit,
                    "radius": radius,
                    "state": state,
                    "street": street,
                    "sub_district": sub_district,
                },
                multigeocode_search_params.MultigeocodeSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, multigeocode_search_params.MultigeocodeSearchParams),
            ),
            cast_to=MultigeocodeSearchResponse,
        )


class MultigeocodeResourceWithRawResponse:
    def __init__(self, multigeocode: MultigeocodeResource) -> None:
        self._multigeocode = multigeocode

        self.search = to_raw_response_wrapper(
            multigeocode.search,
        )

    @cached_property
    def place(self) -> PlaceResourceWithRawResponse:
        return PlaceResourceWithRawResponse(self._multigeocode.place)


class AsyncMultigeocodeResourceWithRawResponse:
    def __init__(self, multigeocode: AsyncMultigeocodeResource) -> None:
        self._multigeocode = multigeocode

        self.search = async_to_raw_response_wrapper(
            multigeocode.search,
        )

    @cached_property
    def place(self) -> AsyncPlaceResourceWithRawResponse:
        return AsyncPlaceResourceWithRawResponse(self._multigeocode.place)


class MultigeocodeResourceWithStreamingResponse:
    def __init__(self, multigeocode: MultigeocodeResource) -> None:
        self._multigeocode = multigeocode

        self.search = to_streamed_response_wrapper(
            multigeocode.search,
        )

    @cached_property
    def place(self) -> PlaceResourceWithStreamingResponse:
        return PlaceResourceWithStreamingResponse(self._multigeocode.place)


class AsyncMultigeocodeResourceWithStreamingResponse:
    def __init__(self, multigeocode: AsyncMultigeocodeResource) -> None:
        self._multigeocode = multigeocode

        self.search = async_to_streamed_response_wrapper(
            multigeocode.search,
        )

    @cached_property
    def place(self) -> AsyncPlaceResourceWithStreamingResponse:
        return AsyncPlaceResourceWithStreamingResponse(self._multigeocode.place)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["Access"]


class Access(BaseModel):
    lat: Optional[float] = None
    """The latitude of the access point of the search result."""

    lng: Optional[float] = None
    """The longitude of the access point of the search result."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RestrictionSetStateParams"]


class RestrictionSetStateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    state: Required[Literal["`enabled`", "`disabled`", "`deleted`"]]
    """Use this field to specify the new `state` of the restriction.

    Please note that this method cannot update the state of restrictions that are
    currently in 'deleted' state.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["JsonRetrieveResponse", "Row", "RowElement"]


class RowElement(BaseModel):
    distance: Optional[float] = None
    """Distance of the route from an origin to a destination, in meters."""

    duration: Optional[float] = None
    """Duration of the trip from an origin to a destination, in seconds."""


class Row(BaseModel):
    elements: Optional[List[RowElement]] = None
    """An array of objects.

    Each `elements` array corresponds to a single `origins` coordinate and contains
    objects with `distance` and `duration` values for each of the `destinations`.
    The details in the first `elements` array correspond to the first `origins`
    point and the first object corresponds to the first `destinations` point and so
    on.
    """


class JsonRetrieveResponse(BaseModel):
    msg: Optional[str] = None
    """Displays the error message in case of a failed request or operation.

    Please note that this parameter is not returned in the response in case of a
    successful request.
    """

    rows: Optional[List[Row]] = None
    """Container object for a response with an array of arrays structure."""

    status: Optional[str] = None
    """A string indicating the state of the response.

    On normal responses, the value will be `Ok`. Indicative HTTP error codes are
    returned for different errors. See the [API Errors Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["Location"]


class Location(BaseModel):
    lat: float
    """Latitude of location."""

    lon: float
    """Longitude of location."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo
from .place_item_param import PlaceItemParam

__all__ = ["PlaceUpdateParams", "DataSource"]


class PlaceUpdateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    data_source: Annotated[DataSource, PropertyInfo(alias="dataSource")]
    """
    dataSource values can be updated to enhance or prioritize the search results to
    better suit specific business use cases.
    """

    place: Iterable[PlaceItemParam]
    """
    This parameter represents the place details, including geographical information,
    address and other related information.
    """

    score: int
    """Search score of the place.

    This is calculated based on how ‘richly’ the place is defined. For instance, a
    place with street name, city, state and country attributes set might be ranked
    lower than a place which has values of house, building, street name, city, state
    and country attributes set. The score determines the rank of the place among
    search results. You can also use this field to set a custom score as per its
    relevance to rank it among the search results from multiple data sources.
    """


class DataSource(TypedDict, total=False):
    ref_id: Annotated[str, PropertyInfo(alias="refId")]
    """
    This parameter represents the unique reference ID associated with the data
    source.
    """

    source: str
    """1.

    Move the place to a new dataset by setting the value to a unique dataset name.
    You can also move the place to an existing dataset by using an existing dataset
    name other than the current one. In both cases, the current datasource will be
    replaced for the specified docID.

    2. Update the place in an existing dataset by setting the name to the current
       value.
    """

    status: Literal["enable", "disable"]
    """
    Set this to either `enable` or `disable` to allow the place to be retrieved by a
    search API or block it respectively.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AssetUpdateAttributesParams"]


class AssetUpdateAttributesParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    attributes: Required[object]
    """
    `attributes` can be used to add any useful information or context to your assets
    like the vehicle type, shift timing etc. These attributes can also be used to
    filter assets in **Search**, **Monitor**, and _Get Asset List_ queries.

    Provide the attributes to be added or updated, in `key`:`value` format. If an
    existing `key` is provided in the input, then the `value` will be modified as
    per the input value. If a new `key` is provided in the input, then the `key`
    would be added to the existing set. The contents of any `value` field are
    neither altered nor removed unless specifically referred to by its `key` in the
    input request.

    Please note that the maximum number of `key`:`value` pairs that can be added to
    an `attributes` object is 100. Also, the overall size of `attributes` object
    should not exceed 65kb.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import (
    MdmCreateDistanceMatrixResponse,
    MdmGetDistanceMatrixStatusResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMdm:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_distance_matrix(self, client: NextbillionSDK) -> None:
        mdm = client.mdm.create_distance_matrix(
            key="key=API_KEY",
            option="flexible",
            origins="origins",
        )
        assert_matches_type(MdmCreateDistanceMatrixResponse, mdm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_distance_matrix_with_all_params(self, client: NextbillionSDK) -> None:
        mdm = client.mdm.create_distance_matrix(
            key="key=API_KEY",
            option="flexible",
            origins="origins",
            spliter="od_number_spliter",
            area="singapore",
            avoid="`toll`",
            cross_border=True,
            departure_time=0,
            destinations="destinations",
            destinations_approach="`unrestricted`",
            hazmat_type="`general`",
            mode="`car`",
            origins_approach="`unrestricted`",
            route_type="`fastest`",
            truck_axle_load=0,
            truck_size='"truck_size"=200,210,600',
            truck_weight=0,
        )
        assert_matches_type(MdmCreateDistanceMatrixResponse, mdm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_distance_matrix(self, client: NextbillionSDK) -> None:
        response = client.mdm.with_raw_response.create_distance_matrix(
            key="key=API_KEY",
            option="flexible",
            origins="origins",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdm = response.parse()
        assert_matches_type(MdmCreateDistanceMatrixResponse, mdm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_distance_matrix(self, client: NextbillionSDK) -> None:
        with client.mdm.with_streaming_response.create_distance_matrix(
            key="key=API_KEY",
            option="flexible",
            origins="origins",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdm = response.parse()
            assert_matches_type(MdmCreateDistanceMatrixResponse, mdm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get_distance_matrix_status(self, client: NextbillionSDK) -> None:
        mdm = client.mdm.get_distance_matrix_status(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(MdmGetDistanceMatrixStatusResponse, mdm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_distance_matrix_status(self, client: NextbillionSDK) -> None:
        response = client.mdm.with_raw_response.get_distance_matrix_status(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdm = response.parse()
        assert_matches_type(MdmGetDistanceMatrixStatusResponse, mdm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_distance_matrix_status(self, client: NextbillionSDK) -> None:
        with client.mdm.with_streaming_response.get_distance_matrix_status(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdm = response.parse()
            assert_matches_type(MdmGetDistanceMatrixStatusResponse, mdm, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncMdm:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_distance_matrix(self, async_client: AsyncNextbillionSDK) -> None:
        mdm = await async_client.mdm.create_distance_matrix(
            key="key=API_KEY",
            option="flexible",
            origins="origins",
        )
        assert_matches_type(MdmCreateDistanceMatrixResponse, mdm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_distance_matrix_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        mdm = await async_client.mdm.create_distance_matrix(
            key="key=API_KEY",
            option="flexible",
            origins="origins",
            spliter="od_number_spliter",
            area="singapore",
            avoid="`toll`",
            cross_border=True,
            departure_time=0,
            destinations="destinations",
            destinations_approach="`unrestricted`",
            hazmat_type="`general`",
            mode="`car`",
            origins_approach="`unrestricted`",
            route_type="`fastest`",
            truck_axle_load=0,
            truck_size='"truck_size"=200,210,600',
            truck_weight=0,
        )
        assert_matches_type(MdmCreateDistanceMatrixResponse, mdm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_distance_matrix(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.mdm.with_raw_response.create_distance_matrix(
            key="key=API_KEY",
            option="flexible",
            origins="origins",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdm = await response.parse()
        assert_matches_type(MdmCreateDistanceMatrixResponse, mdm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_distance_matrix(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.mdm.with_streaming_response.create_distance_matrix(
            key="key=API_KEY",
            option="flexible",
            origins="origins",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdm = await response.parse()
            assert_matches_type(MdmCreateDistanceMatrixResponse, mdm, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_distance_matrix_status(self, async_client: AsyncNextbillionSDK) -> None:
        mdm = await async_client.mdm.get_distance_matrix_status(
            id="id",
            key="key=API_KEY",
        )
        assert_matches_type(MdmGetDistanceMatrixStatusResponse, mdm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_distance_matrix_status(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.mdm.with_raw_response.get_distance_matrix_status(
            id="id",
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdm = await response.parse()
        assert_matches_type(MdmGetDistanceMatrixStatusResponse, mdm, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_distance_matrix_status(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.mdm.with_streaming_response.get_distance_matrix_status(
            id="id",
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdm = await response.parse()
            assert_matches_type(MdmGetDistanceMatrixStatusResponse, mdm, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Required, TypedDict

from .optimization.job_param import JobParam
from .optimization.vehicle_param import VehicleParam
from .optimization.shipment_param import ShipmentParam

__all__ = ["OptimizationReOptimizeParams", "JobChanges", "ShipmentChanges", "VehicleChanges"]


class OptimizationReOptimizeParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    existing_request_id: Required[str]
    """Specify the unique request ID that needs to be re-optimized."""

    job_changes: JobChanges
    """
    This section gathers information on modifications to the number of jobs or their
    individual requirements for re-optimization. Any job from the original solution
    not specified here will be re-planned without alteration during the
    re-optimization process.
    """

    locations: List[str]
    """Provide the list of locations to be used during re-optimization process.

    Please note that

    - Providing the location input overwrites the list of locations used in the
      original request.
    - The location_indexes associated with all tasks and vehicles (both from the
      original and new re-optimization input requests) will follow the updated list
      of locations.

    As a best practice:

    1.  Don't provide the `locations` input when re-optimizing, if the original set
        contains all the required location coordinates.
    2.  If any new location coordinates are required for re-optimization, copy the
        full, original location list and update it in the following manner before
        adding it to the re-optimization input:

        1.  Ensure to not update the indexes of locations which just need to be
            "modified".
        2.  Add new location coordinates towards the end of the list.
    """

    shipment_changes: ShipmentChanges
    """
    This section gathers information on modifications to the number of shipments or
    their individual requirements for re-optimization. Any shipment from the
    original solution not specified here will be re-planned without alteration
    during the re-optimization process.
    """

    vehicle_changes: VehicleChanges
    """
    This section gathers information on modifications to the number of vehicles or
    individual vehicle configurations for re-optimizing an existing solution. Any
    vehicle from the original solution not specified here will be reused without
    alteration during the re-optimization process.
    """


class JobChanges(TypedDict, total=False):
    add: Iterable[JobParam]
    """
    An array of objects to collect the details of the new jobs to be added during
    re-optimization. Each object represents one job. Please make sure the IDs
    provided for new jobs are unique with respect to the IDs of the jobs in the
    original request.
    """

    modify: Iterable[JobParam]
    """
    An array of objects to collect the modified details of existing jobs used in the
    original request. Each object represents one job. Please make sure all the job
    IDs provided here are same as the ones in the original request.
    """

    remove: List[str]
    """An array of job IDs to be removed when during re-optimization.

    All job IDs provided must have been part of the original request.
    """


class ShipmentChanges(TypedDict, total=False):
    add: Iterable[ShipmentParam]
    """
    An array of objects to collect the details of the new shipments to be added
    during re-optimization. Each object represents one shipment. Please make sure
    the IDs provided for new shipments are unique with respect to the IDs of the
    shipments in the original request.
    """

    modify: Iterable[ShipmentParam]
    """
    An array of objects to collect the modified details of existing shipments used
    in the original request. Each object represents one shipment. Please make sure
    all the shipment IDs provided here are same as the ones in the original request.
    """

    remove: List[str]
    """An array of shipment IDs to be removed when during re-optimization.

    All shipment IDs provided must have been part of the original request.
    """


class VehicleChanges(TypedDict, total=False):
    add: Iterable[VehicleParam]
    """
    An array of objects to collect the details of the new vehicles to be added for
    re-optimization. Each object represents one vehicle. Please make sure the IDs
    provided for new vehicles are unique with respect to the IDs of the vehicles in
    the original request.
    """

    modify: VehicleParam

    remove: List[str]
    """An array of vehicle IDs to be removed when during re-optimization.

    All vehicle IDs provided must have been part of the original request.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import restrictions_item_list_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.restrictions_item_list_response import RestrictionsItemListResponse

__all__ = ["RestrictionsItemsResource", "AsyncRestrictionsItemsResource"]


class RestrictionsItemsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RestrictionsItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return RestrictionsItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RestrictionsItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return RestrictionsItemsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        max_lat: float,
        max_lon: float,
        min_lat: float,
        min_lon: float,
        group_id: float | NotGiven = NOT_GIVEN,
        mode: Literal["0w", "1w", "2w", "3w", "4w", "6w"] | NotGiven = NOT_GIVEN,
        restriction_type: Literal["turn", "parking", "fixedspeed", "maxspeed", "closure", "truck"]
        | NotGiven = NOT_GIVEN,
        source: str | NotGiven = NOT_GIVEN,
        state: Literal["enabled", "disabled", "deleted"] | NotGiven = NOT_GIVEN,
        status: Literal["active", "inactive"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RestrictionsItemListResponse:
        """
        Get restriction items by bbox

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/restrictions_items",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "max_lat": max_lat,
                        "max_lon": max_lon,
                        "min_lat": min_lat,
                        "min_lon": min_lon,
                        "group_id": group_id,
                        "mode": mode,
                        "restriction_type": restriction_type,
                        "source": source,
                        "state": state,
                        "status": status,
                    },
                    restrictions_item_list_params.RestrictionsItemListParams,
                ),
            ),
            cast_to=RestrictionsItemListResponse,
        )


class AsyncRestrictionsItemsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRestrictionsItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRestrictionsItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRestrictionsItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncRestrictionsItemsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        max_lat: float,
        max_lon: float,
        min_lat: float,
        min_lon: float,
        group_id: float | NotGiven = NOT_GIVEN,
        mode: Literal["0w", "1w", "2w", "3w", "4w", "6w"] | NotGiven = NOT_GIVEN,
        restriction_type: Literal["turn", "parking", "fixedspeed", "maxspeed", "closure", "truck"]
        | NotGiven = NOT_GIVEN,
        source: str | NotGiven = NOT_GIVEN,
        state: Literal["enabled", "disabled", "deleted"] | NotGiven = NOT_GIVEN,
        status: Literal["active", "inactive"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RestrictionsItemListResponse:
        """
        Get restriction items by bbox

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/restrictions_items",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "max_lat": max_lat,
                        "max_lon": max_lon,
                        "min_lat": min_lat,
                        "min_lon": min_lon,
                        "group_id": group_id,
                        "mode": mode,
                        "restriction_type": restriction_type,
                        "source": source,
                        "state": state,
                        "status": status,
                    },
                    restrictions_item_list_params.RestrictionsItemListParams,
                ),
            ),
            cast_to=RestrictionsItemListResponse,
        )


class RestrictionsItemsResourceWithRawResponse:
    def __init__(self, restrictions_items: RestrictionsItemsResource) -> None:
        self._restrictions_items = restrictions_items

        self.list = to_raw_response_wrapper(
            restrictions_items.list,
        )


class AsyncRestrictionsItemsResourceWithRawResponse:
    def __init__(self, restrictions_items: AsyncRestrictionsItemsResource) -> None:
        self._restrictions_items = restrictions_items

        self.list = async_to_raw_response_wrapper(
            restrictions_items.list,
        )


class RestrictionsItemsResourceWithStreamingResponse:
    def __init__(self, restrictions_items: RestrictionsItemsResource) -> None:
        self._restrictions_items = restrictions_items

        self.list = to_streamed_response_wrapper(
            restrictions_items.list,
        )


class AsyncRestrictionsItemsResourceWithStreamingResponse:
    def __init__(self, restrictions_items: AsyncRestrictionsItemsResource) -> None:
        self._restrictions_items = restrictions_items

        self.list = async_to_streamed_response_wrapper(
            restrictions_items.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["PlaceDeleteResponse"]


class PlaceDeleteResponse(BaseModel):
    msg: Optional[str] = None
    """
    This could be “Ok” representing success or “not found” representing error in
    processing the request.
    """

    status: Optional[str] = None
    """Represents the status of the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel
from .track_location import TrackLocation

__all__ = ["LocationGetLastResponse", "Data"]


class Data(BaseModel):
    location: Optional[TrackLocation] = None
    """An object with details of the tracked location.

    Please note that if there are no tracking records for an asset, no location data
    will be returned.
    """


class LocationGetLastResponse(BaseModel):
    data: Optional[Data] = None
    """
    An object containing the information about the last tracked location of the
    requested `asset`.
    """

    message: Optional[str] = None
    """Displays the error message in case of a failed request.

    If the request is successful, this field is not present in the response.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On successful responses, the value will be `Ok`. Indicative error messages are
    returned for different errors. See the [API Error Codes](#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["LookupByIDParams"]


class LookupByIDParams(TypedDict, total=False):
    id: Required[str]
    """
    Specify the unique identifier of a specific POI, Street, Geography, Point
    Address or other entities to retrieve its details.
    """

    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["SkynetSubscribeResponse"]


class SkynetSubscribeResponse(BaseModel):
    id: Optional[str] = None
    """Subscription ID as provided in the input action message."""

    error: Optional[str] = None
    """Returns the error message when `status: error`.

    Otherwise, response doesn't contain this field.
    """

    status: Optional[str] = None
    """Status of the action. It can have only two values - "success" or "error"."""

    timestamp: Optional[int] = None
    """
    Returns the UNIX timestamp, in milliseconds format, when the web-socket returns
    the action response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TripEndParams"]


class TripEndParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    id: Required[str]
    """Specify the ID of the trip to be ended."""

    cluster: Literal["america"]
    """the cluster of the region you want to use"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from nextbillion_sdk import NextbillionSDK, AsyncNextbillionSDK
from nextbillion_sdk.types import (
    RichGroupResponse,
    RestrictionListResponse,
    RestrictionDeleteResponse,
    RestrictionListByBboxResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRestrictions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.create(
            restriction_type="turn",
            key="key=API_KEY",
            area="area",
            name="name",
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.create(
            restriction_type="turn",
            key="key=API_KEY",
            area="area",
            name="name",
            latlon=True,
            comment="comment",
            direction="`forward`",
            end_time=0,
            geofence=[[0]],
            height=0,
            length=0,
            mode=["0w"],
            repeat_on='repeatOn="Mo-Fr 07:00-09:00,17:00-19:00"',
            segments=[
                {
                    "from": 0,
                    "to": 0,
                }
            ],
            speed=0,
            speed_limit=0,
            start_time=0,
            tracks=[[0]],
            turns=[
                {
                    "from": 0,
                    "to": 0,
                    "via": 0,
                }
            ],
            weight=0,
            width=0,
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: NextbillionSDK) -> None:
        response = client.restrictions.with_raw_response.create(
            restriction_type="turn",
            key="key=API_KEY",
            area="area",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = response.parse()
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: NextbillionSDK) -> None:
        with client.restrictions.with_streaming_response.create(
            restriction_type="turn",
            key="key=API_KEY",
            area="area",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = response.parse()
            assert_matches_type(RichGroupResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.retrieve(
            id=0,
            key="key=API_KEY",
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_with_all_params(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.retrieve(
            id=0,
            key="key=API_KEY",
            transform=True,
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: NextbillionSDK) -> None:
        response = client.restrictions.with_raw_response.retrieve(
            id=0,
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = response.parse()
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: NextbillionSDK) -> None:
        with client.restrictions.with_streaming_response.retrieve(
            id=0,
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = response.parse()
            assert_matches_type(RichGroupResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.update(
            id=0,
            key="key=API_KEY",
            area="area",
            name="name",
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.update(
            id=0,
            key="key=API_KEY",
            area="area",
            name="name",
            latlon=True,
            comment="comment",
            direction="`forward`",
            end_time=0,
            geofence=[[0]],
            height=0,
            length=0,
            mode=["0w"],
            repeat_on='repeatOn="Mo-Fr 07:00-09:00,17:00-19:00"',
            segments=[
                {
                    "from": 0,
                    "to": 0,
                }
            ],
            speed=0,
            speed_limit=0,
            start_time=0,
            tracks=[[0]],
            turns=[
                {
                    "from": 0,
                    "to": 0,
                    "via": 0,
                }
            ],
            weight=0,
            width=0,
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: NextbillionSDK) -> None:
        response = client.restrictions.with_raw_response.update(
            id=0,
            key="key=API_KEY",
            area="area",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = response.parse()
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: NextbillionSDK) -> None:
        with client.restrictions.with_streaming_response.update(
            id=0,
            key="key=API_KEY",
            area="area",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = response.parse()
            assert_matches_type(RichGroupResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.list(
            area="area",
            key="key=API_KEY",
            limit=0,
            offset=0,
        )
        assert_matches_type(RestrictionListResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.list(
            area="area",
            key="key=API_KEY",
            limit=0,
            offset=0,
            mode="`0w`",
            name="name",
            restriction_type="`turn`",
            source="rrt",
            state="`enabled`",
            status="`active`",
            transform=True,
        )
        assert_matches_type(RestrictionListResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: NextbillionSDK) -> None:
        response = client.restrictions.with_raw_response.list(
            area="area",
            key="key=API_KEY",
            limit=0,
            offset=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = response.parse()
        assert_matches_type(RestrictionListResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: NextbillionSDK) -> None:
        with client.restrictions.with_streaming_response.list(
            area="area",
            key="key=API_KEY",
            limit=0,
            offset=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = response.parse()
            assert_matches_type(RestrictionListResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.delete(
            id=0,
            key="key=API_KEY",
        )
        assert_matches_type(RestrictionDeleteResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: NextbillionSDK) -> None:
        response = client.restrictions.with_raw_response.delete(
            id=0,
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = response.parse()
        assert_matches_type(RestrictionDeleteResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: NextbillionSDK) -> None:
        with client.restrictions.with_streaming_response.delete(
            id=0,
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = response.parse()
            assert_matches_type(RestrictionDeleteResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list_by_bbox(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.list_by_bbox(
            key="key=API_KEY",
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        )
        assert_matches_type(RestrictionListByBboxResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_by_bbox_with_all_params(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.list_by_bbox(
            key="key=API_KEY",
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
            mode=["0w"],
            restriction_type="turn",
            source="rrt",
            state="`enabled`",
            status="`active`",
            transform=True,
        )
        assert_matches_type(RestrictionListByBboxResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list_by_bbox(self, client: NextbillionSDK) -> None:
        response = client.restrictions.with_raw_response.list_by_bbox(
            key="key=API_KEY",
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = response.parse()
        assert_matches_type(RestrictionListByBboxResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list_by_bbox(self, client: NextbillionSDK) -> None:
        with client.restrictions.with_streaming_response.list_by_bbox(
            key="key=API_KEY",
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = response.parse()
            assert_matches_type(RestrictionListByBboxResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_set_state(self, client: NextbillionSDK) -> None:
        restriction = client.restrictions.set_state(
            id=0,
            key="key=API_KEY",
            state="`enabled`",
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_set_state(self, client: NextbillionSDK) -> None:
        response = client.restrictions.with_raw_response.set_state(
            id=0,
            key="key=API_KEY",
            state="`enabled`",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = response.parse()
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_set_state(self, client: NextbillionSDK) -> None:
        with client.restrictions.with_streaming_response.set_state(
            id=0,
            key="key=API_KEY",
            state="`enabled`",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = response.parse()
            assert_matches_type(RichGroupResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncRestrictions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.create(
            restriction_type="turn",
            key="key=API_KEY",
            area="area",
            name="name",
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.create(
            restriction_type="turn",
            key="key=API_KEY",
            area="area",
            name="name",
            latlon=True,
            comment="comment",
            direction="`forward`",
            end_time=0,
            geofence=[[0]],
            height=0,
            length=0,
            mode=["0w"],
            repeat_on='repeatOn="Mo-Fr 07:00-09:00,17:00-19:00"',
            segments=[
                {
                    "from": 0,
                    "to": 0,
                }
            ],
            speed=0,
            speed_limit=0,
            start_time=0,
            tracks=[[0]],
            turns=[
                {
                    "from": 0,
                    "to": 0,
                    "via": 0,
                }
            ],
            weight=0,
            width=0,
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.restrictions.with_raw_response.create(
            restriction_type="turn",
            key="key=API_KEY",
            area="area",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = await response.parse()
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.restrictions.with_streaming_response.create(
            restriction_type="turn",
            key="key=API_KEY",
            area="area",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = await response.parse()
            assert_matches_type(RichGroupResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.retrieve(
            id=0,
            key="key=API_KEY",
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.retrieve(
            id=0,
            key="key=API_KEY",
            transform=True,
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.restrictions.with_raw_response.retrieve(
            id=0,
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = await response.parse()
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.restrictions.with_streaming_response.retrieve(
            id=0,
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = await response.parse()
            assert_matches_type(RichGroupResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.update(
            id=0,
            key="key=API_KEY",
            area="area",
            name="name",
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.update(
            id=0,
            key="key=API_KEY",
            area="area",
            name="name",
            latlon=True,
            comment="comment",
            direction="`forward`",
            end_time=0,
            geofence=[[0]],
            height=0,
            length=0,
            mode=["0w"],
            repeat_on='repeatOn="Mo-Fr 07:00-09:00,17:00-19:00"',
            segments=[
                {
                    "from": 0,
                    "to": 0,
                }
            ],
            speed=0,
            speed_limit=0,
            start_time=0,
            tracks=[[0]],
            turns=[
                {
                    "from": 0,
                    "to": 0,
                    "via": 0,
                }
            ],
            weight=0,
            width=0,
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.restrictions.with_raw_response.update(
            id=0,
            key="key=API_KEY",
            area="area",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = await response.parse()
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.restrictions.with_streaming_response.update(
            id=0,
            key="key=API_KEY",
            area="area",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = await response.parse()
            assert_matches_type(RichGroupResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.list(
            area="area",
            key="key=API_KEY",
            limit=0,
            offset=0,
        )
        assert_matches_type(RestrictionListResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.list(
            area="area",
            key="key=API_KEY",
            limit=0,
            offset=0,
            mode="`0w`",
            name="name",
            restriction_type="`turn`",
            source="rrt",
            state="`enabled`",
            status="`active`",
            transform=True,
        )
        assert_matches_type(RestrictionListResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.restrictions.with_raw_response.list(
            area="area",
            key="key=API_KEY",
            limit=0,
            offset=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = await response.parse()
        assert_matches_type(RestrictionListResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.restrictions.with_streaming_response.list(
            area="area",
            key="key=API_KEY",
            limit=0,
            offset=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = await response.parse()
            assert_matches_type(RestrictionListResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.delete(
            id=0,
            key="key=API_KEY",
        )
        assert_matches_type(RestrictionDeleteResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.restrictions.with_raw_response.delete(
            id=0,
            key="key=API_KEY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = await response.parse()
        assert_matches_type(RestrictionDeleteResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.restrictions.with_streaming_response.delete(
            id=0,
            key="key=API_KEY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = await response.parse()
            assert_matches_type(RestrictionDeleteResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_by_bbox(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.list_by_bbox(
            key="key=API_KEY",
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        )
        assert_matches_type(RestrictionListByBboxResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_by_bbox_with_all_params(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.list_by_bbox(
            key="key=API_KEY",
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
            mode=["0w"],
            restriction_type="turn",
            source="rrt",
            state="`enabled`",
            status="`active`",
            transform=True,
        )
        assert_matches_type(RestrictionListByBboxResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list_by_bbox(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.restrictions.with_raw_response.list_by_bbox(
            key="key=API_KEY",
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = await response.parse()
        assert_matches_type(RestrictionListByBboxResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list_by_bbox(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.restrictions.with_streaming_response.list_by_bbox(
            key="key=API_KEY",
            max_lat=0,
            max_lon=0,
            min_lat=0,
            min_lon=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = await response.parse()
            assert_matches_type(RestrictionListByBboxResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_set_state(self, async_client: AsyncNextbillionSDK) -> None:
        restriction = await async_client.restrictions.set_state(
            id=0,
            key="key=API_KEY",
            state="`enabled`",
        )
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_set_state(self, async_client: AsyncNextbillionSDK) -> None:
        response = await async_client.restrictions.with_raw_response.set_state(
            id=0,
            key="key=API_KEY",
            state="`enabled`",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        restriction = await response.parse()
        assert_matches_type(RichGroupResponse, restriction, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_set_state(self, async_client: AsyncNextbillionSDK) -> None:
        async with async_client.restrictions.with_streaming_response.set_state(
            id=0,
            key="key=API_KEY",
            state="`enabled`",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            restriction = await response.parse()
            assert_matches_type(RichGroupResponse, restriction, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from .meta_data_param import MetaDataParam

__all__ = ["AssetCreateParams"]


class AssetCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""

    attributes: object
    """
    `attributes` can be used to store custom information about an asset in
    `key`:`value` format. Use `attributes` to add any useful information or context
    to your assets like the vehicle type, shift timing etc. Moreover, these
    attributes can be used to filter `assets` in **Search**, **Monitor**, and _Get
    Asset List_ queries.

    Please note that the maximum number of `key`:`value` pairs that can be added to
    an `attributes` object is 100. Also, the overall size of `attributes` object
    should not exceed 65kb.
    """

    custom_id: str
    """Set a unique ID for the new `asset`.

    If not provided, an ID will be automatically generated in UUID format. A valid
    `custom_id` can contain letters, numbers, "-", & "\\__" only.

    Please note that the ID of an `asset` can not be changed once it is created.
    """

    description: str
    """Description for the `asset`."""

    meta_data: MetaDataParam
    """Any valid json object data.

    Can be used to save customized data. Max size is 65kb.
    """

    name: str
    """Name of the `asset`.

    Use this field to assign a meaningful, custom name to the `asset` being created.
    """

    tags: List[str]
    """
    **This parameter will be deprecated soon! Please use the `attributes` parameter
    to add labels or markers for the asset.**

    Tags of the `asset`. `tags` can be used for filtering assets in operations like
    _Get Asset List_ and asset **Search** methods. They can also be used for
    monitoring of assets using the **Monitor** methods after linking `tags` and
    `asset`.

    Valid `tags` are strings consisting of alphanumeric characters (A-Z, a-z, 0-9)
    along with the underscore ('\\__') and hyphen ('-') symbols.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.multigeocode import place_create_params, place_delete_params, place_update_params, place_retrieve_params
from ...types.multigeocode.place_item_param import PlaceItemParam
from ...types.multigeocode.place_create_response import PlaceCreateResponse
from ...types.multigeocode.place_delete_response import PlaceDeleteResponse
from ...types.multigeocode.place_update_response import PlaceUpdateResponse
from ...types.multigeocode.place_retrieve_response import PlaceRetrieveResponse

__all__ = ["PlaceResource", "AsyncPlaceResource"]


class PlaceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PlaceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PlaceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlaceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return PlaceResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        key: str,
        place: Iterable[place_create_params.Place],
        data_source: place_create_params.DataSource | NotGiven = NOT_GIVEN,
        force: bool | NotGiven = NOT_GIVEN,
        score: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlaceCreateResponse:
        """
        The "Add Place" method allows users to create custom places

        Add place method provides the flexibility to create custom places in a way that
        suits your business needs. The newly created place and its attributes can be
        added to custom (proprietary) dataset - to the effect of building your own
        places dataset (s) - or, to a default dataset. Overcome inaccurate ‘POI’ details
        from default search provider by creating custom, highly accurate ‘POIs’.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          place: This parameter represents the place details, including geographical information,
              address and other related information.

          data_source: It contains information about the dataset that returns the specific result

          force: When 2 places are located within 100 meters of each other and have more than 90%
              of matching attributes (at least 11 out of 12 attributes in the “place” object),
              they will be considered duplicates and any requests to add such a new place
              would be rejected. Set `force=true` to override this duplicate check. You can
              use this to create closely located POIs. For instance, places inside a mall,
              university or a government building etc.

          score: Search score of the place. This is calculated based on how ‘richly’ the place is
              defined. For instance, a place with - street name, city, state and country
              attributes set might be ranked lower than a place which has values of - house,
              building, street name, city, state and country attributes set. The score
              determines the rank of the place among search results. You can also use this
              field to set a custom score as per its relevance to rank it among the search
              results from multiple data sources.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/multigeocode/place",
            body=maybe_transform(
                {
                    "place": place,
                    "data_source": data_source,
                    "force": force,
                    "score": score,
                },
                place_create_params.PlaceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, place_create_params.PlaceCreateParams),
            ),
            cast_to=PlaceCreateResponse,
        )

    def retrieve(
        self,
        doc_id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlaceRetrieveResponse:
        """
        Use this method to get the details of previously created custom places using its
        NextBillion ID.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not doc_id:
            raise ValueError(f"Expected a non-empty value for `doc_id` but received {doc_id!r}")
        return self._get(
            f"/multigeocode/place/{doc_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, place_retrieve_params.PlaceRetrieveParams),
            ),
            cast_to=PlaceRetrieveResponse,
        )

    def update(
        self,
        doc_id: str,
        *,
        key: str,
        data_source: place_update_params.DataSource | NotGiven = NOT_GIVEN,
        place: Iterable[PlaceItemParam] | NotGiven = NOT_GIVEN,
        score: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlaceUpdateResponse:
        """
        The "Update Place" method allows businesses to update the attributes of an
        existing place.

        This method allows you to update the attributes of custom places. In effect,
        updating a place replaces the current information in search results with the
        updated information associated with the specific docID. Use this method to
        enhance the accuracy/usability of your search results with respect to the
        default dataset to suit your business needs.

        If you want to prioritize a particular result in your search results, update the
        ‘score’ of that specific place.
        Alternatively, you can block places which are no longer needed by setting their
        status: ‘disable’.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          data_source: dataSource values can be updated to enhance or prioritize the search results to
              better suit specific business use cases.

          place: This parameter represents the place details, including geographical information,
              address and other related information.

          score: Search score of the place. This is calculated based on how ‘richly’ the place is
              defined. For instance, a place with street name, city, state and country
              attributes set might be ranked lower than a place which has values of house,
              building, street name, city, state and country attributes set. The score
              determines the rank of the place among search results. You can also use this
              field to set a custom score as per its relevance to rank it among the search
              results from multiple data sources.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not doc_id:
            raise ValueError(f"Expected a non-empty value for `doc_id` but received {doc_id!r}")
        return self._put(
            f"/multigeocode/place/{doc_id}",
            body=maybe_transform(
                {
                    "data_source": data_source,
                    "place": place,
                    "score": score,
                },
                place_update_params.PlaceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, place_update_params.PlaceUpdateParams),
            ),
            cast_to=PlaceUpdateResponse,
        )

    def delete(
        self,
        doc_id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlaceDeleteResponse:
        """
        The "Delete Place" method enables businesses to delete a previously created
        place

        Use this method to delete a previously created place. Please note that the place
        associated with the specified docID only would be deleted. As a result, once a
        place is deleted, the search API can still return valid results from the default
        datasets or others, if present.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not doc_id:
            raise ValueError(f"Expected a non-empty value for `doc_id` but received {doc_id!r}")
        return self._delete(
            f"/multigeocode/place/{doc_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, place_delete_params.PlaceDeleteParams),
            ),
            cast_to=PlaceDeleteResponse,
        )


class AsyncPlaceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPlaceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPlaceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlaceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncPlaceResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        key: str,
        place: Iterable[place_create_params.Place],
        data_source: place_create_params.DataSource | NotGiven = NOT_GIVEN,
        force: bool | NotGiven = NOT_GIVEN,
        score: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlaceCreateResponse:
        """
        The "Add Place" method allows users to create custom places

        Add place method provides the flexibility to create custom places in a way that
        suits your business needs. The newly created place and its attributes can be
        added to custom (proprietary) dataset - to the effect of building your own
        places dataset (s) - or, to a default dataset. Overcome inaccurate ‘POI’ details
        from default search provider by creating custom, highly accurate ‘POIs’.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          place: This parameter represents the place details, including geographical information,
              address and other related information.

          data_source: It contains information about the dataset that returns the specific result

          force: When 2 places are located within 100 meters of each other and have more than 90%
              of matching attributes (at least 11 out of 12 attributes in the “place” object),
              they will be considered duplicates and any requests to add such a new place
              would be rejected. Set `force=true` to override this duplicate check. You can
              use this to create closely located POIs. For instance, places inside a mall,
              university or a government building etc.

          score: Search score of the place. This is calculated based on how ‘richly’ the place is
              defined. For instance, a place with - street name, city, state and country
              attributes set might be ranked lower than a place which has values of - house,
              building, street name, city, state and country attributes set. The score
              determines the rank of the place among search results. You can also use this
              field to set a custom score as per its relevance to rank it among the search
              results from multiple data sources.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/multigeocode/place",
            body=await async_maybe_transform(
                {
                    "place": place,
                    "data_source": data_source,
                    "force": force,
                    "score": score,
                },
                place_create_params.PlaceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, place_create_params.PlaceCreateParams),
            ),
            cast_to=PlaceCreateResponse,
        )

    async def retrieve(
        self,
        doc_id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlaceRetrieveResponse:
        """
        Use this method to get the details of previously created custom places using its
        NextBillion ID.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not doc_id:
            raise ValueError(f"Expected a non-empty value for `doc_id` but received {doc_id!r}")
        return await self._get(
            f"/multigeocode/place/{doc_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, place_retrieve_params.PlaceRetrieveParams),
            ),
            cast_to=PlaceRetrieveResponse,
        )

    async def update(
        self,
        doc_id: str,
        *,
        key: str,
        data_source: place_update_params.DataSource | NotGiven = NOT_GIVEN,
        place: Iterable[PlaceItemParam] | NotGiven = NOT_GIVEN,
        score: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlaceUpdateResponse:
        """
        The "Update Place" method allows businesses to update the attributes of an
        existing place.

        This method allows you to update the attributes of custom places. In effect,
        updating a place replaces the current information in search results with the
        updated information associated with the specific docID. Use this method to
        enhance the accuracy/usability of your search results with respect to the
        default dataset to suit your business needs.

        If you want to prioritize a particular result in your search results, update the
        ‘score’ of that specific place.
        Alternatively, you can block places which are no longer needed by setting their
        status: ‘disable’.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          data_source: dataSource values can be updated to enhance or prioritize the search results to
              better suit specific business use cases.

          place: This parameter represents the place details, including geographical information,
              address and other related information.

          score: Search score of the place. This is calculated based on how ‘richly’ the place is
              defined. For instance, a place with street name, city, state and country
              attributes set might be ranked lower than a place which has values of house,
              building, street name, city, state and country attributes set. The score
              determines the rank of the place among search results. You can also use this
              field to set a custom score as per its relevance to rank it among the search
              results from multiple data sources.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not doc_id:
            raise ValueError(f"Expected a non-empty value for `doc_id` but received {doc_id!r}")
        return await self._put(
            f"/multigeocode/place/{doc_id}",
            body=await async_maybe_transform(
                {
                    "data_source": data_source,
                    "place": place,
                    "score": score,
                },
                place_update_params.PlaceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, place_update_params.PlaceUpdateParams),
            ),
            cast_to=PlaceUpdateResponse,
        )

    async def delete(
        self,
        doc_id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PlaceDeleteResponse:
        """
        The "Delete Place" method enables businesses to delete a previously created
        place

        Use this method to delete a previously created place. Please note that the place
        associated with the specified docID only would be deleted. As a result, once a
        place is deleted, the search API can still return valid results from the default
        datasets or others, if present.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not doc_id:
            raise ValueError(f"Expected a non-empty value for `doc_id` but received {doc_id!r}")
        return await self._delete(
            f"/multigeocode/place/{doc_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, place_delete_params.PlaceDeleteParams),
            ),
            cast_to=PlaceDeleteResponse,
        )


class PlaceResourceWithRawResponse:
    def __init__(self, place: PlaceResource) -> None:
        self._place = place

        self.create = to_raw_response_wrapper(
            place.create,
        )
        self.retrieve = to_raw_response_wrapper(
            place.retrieve,
        )
        self.update = to_raw_response_wrapper(
            place.update,
        )
        self.delete = to_raw_response_wrapper(
            place.delete,
        )


class AsyncPlaceResourceWithRawResponse:
    def __init__(self, place: AsyncPlaceResource) -> None:
        self._place = place

        self.create = async_to_raw_response_wrapper(
            place.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            place.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            place.update,
        )
        self.delete = async_to_raw_response_wrapper(
            place.delete,
        )


class PlaceResourceWithStreamingResponse:
    def __init__(self, place: PlaceResource) -> None:
        self._place = place

        self.create = to_streamed_response_wrapper(
            place.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            place.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            place.update,
        )
        self.delete = to_streamed_response_wrapper(
            place.delete,
        )


class AsyncPlaceResourceWithStreamingResponse:
    def __init__(self, place: AsyncPlaceResource) -> None:
        self._place = place

        self.create = async_to_streamed_response_wrapper(
            place.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            place.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            place.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            place.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .place_item import PlaceItem as PlaceItem
from .place_item_param import PlaceItemParam as PlaceItemParam
from .place_create_params import PlaceCreateParams as PlaceCreateParams
from .place_delete_params import PlaceDeleteParams as PlaceDeleteParams
from .place_update_params import PlaceUpdateParams as PlaceUpdateParams
from .place_create_response import PlaceCreateResponse as PlaceCreateResponse
from .place_delete_response import PlaceDeleteResponse as PlaceDeleteResponse
from .place_retrieve_params import PlaceRetrieveParams as PlaceRetrieveParams
from .place_update_response import PlaceUpdateResponse as PlaceUpdateResponse
from .place_retrieve_response import PlaceRetrieveResponse as PlaceRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.optimization import driver_assignment_assign_params
from ...types.optimization.vehicle_param import VehicleParam
from ...types.optimization.driver_assignment_assign_response import DriverAssignmentAssignResponse

__all__ = ["DriverAssignmentResource", "AsyncDriverAssignmentResource"]


class DriverAssignmentResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DriverAssignmentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DriverAssignmentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DriverAssignmentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return DriverAssignmentResourceWithStreamingResponse(self)

    def assign(
        self,
        *,
        key: str,
        filter: driver_assignment_assign_params.Filter,
        orders: Iterable[driver_assignment_assign_params.Order],
        vehicles: Iterable[VehicleParam],
        options: driver_assignment_assign_params.Options | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DriverAssignmentAssignResponse:
        """
        Assigns available drivers (vehicles) to open orders based on specified criteria
        and constraints.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          filter: Specify the filtering criterion for the vehicles with respect to each order's
              location. `filter` is a mandatory input for all requests.

          orders: Collects the details of open orders to be fulfilled. Each object represents one
              order. All requests must include `orders` as a mandatory input. A maximum of 200
              orders is allowed per request.

          vehicles: Collects the details of vehicles available to fulfill the orders. Each object
              represents one vehicle. All requests must include `vehicles` as a mandatory
              input. A maximum of 100 vehicles is allowed per request.

          options: Configure the assignment constraints and response settings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/optimization/driver-assignment/v1",
            body=maybe_transform(
                {
                    "filter": filter,
                    "orders": orders,
                    "vehicles": vehicles,
                    "options": options,
                },
                driver_assignment_assign_params.DriverAssignmentAssignParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, driver_assignment_assign_params.DriverAssignmentAssignParams),
            ),
            cast_to=DriverAssignmentAssignResponse,
        )


class AsyncDriverAssignmentResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDriverAssignmentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDriverAssignmentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDriverAssignmentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncDriverAssignmentResourceWithStreamingResponse(self)

    async def assign(
        self,
        *,
        key: str,
        filter: driver_assignment_assign_params.Filter,
        orders: Iterable[driver_assignment_assign_params.Order],
        vehicles: Iterable[VehicleParam],
        options: driver_assignment_assign_params.Options | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DriverAssignmentAssignResponse:
        """
        Assigns available drivers (vehicles) to open orders based on specified criteria
        and constraints.

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          filter: Specify the filtering criterion for the vehicles with respect to each order's
              location. `filter` is a mandatory input for all requests.

          orders: Collects the details of open orders to be fulfilled. Each object represents one
              order. All requests must include `orders` as a mandatory input. A maximum of 200
              orders is allowed per request.

          vehicles: Collects the details of vehicles available to fulfill the orders. Each object
              represents one vehicle. All requests must include `vehicles` as a mandatory
              input. A maximum of 100 vehicles is allowed per request.

          options: Configure the assignment constraints and response settings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/optimization/driver-assignment/v1",
            body=await async_maybe_transform(
                {
                    "filter": filter,
                    "orders": orders,
                    "vehicles": vehicles,
                    "options": options,
                },
                driver_assignment_assign_params.DriverAssignmentAssignParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"key": key}, driver_assignment_assign_params.DriverAssignmentAssignParams
                ),
            ),
            cast_to=DriverAssignmentAssignResponse,
        )


class DriverAssignmentResourceWithRawResponse:
    def __init__(self, driver_assignment: DriverAssignmentResource) -> None:
        self._driver_assignment = driver_assignment

        self.assign = to_raw_response_wrapper(
            driver_assignment.assign,
        )


class AsyncDriverAssignmentResourceWithRawResponse:
    def __init__(self, driver_assignment: AsyncDriverAssignmentResource) -> None:
        self._driver_assignment = driver_assignment

        self.assign = async_to_raw_response_wrapper(
            driver_assignment.assign,
        )


class DriverAssignmentResourceWithStreamingResponse:
    def __init__(self, driver_assignment: DriverAssignmentResource) -> None:
        self._driver_assignment = driver_assignment

        self.assign = to_streamed_response_wrapper(
            driver_assignment.assign,
        )


class AsyncDriverAssignmentResourceWithStreamingResponse:
    def __init__(self, driver_assignment: AsyncDriverAssignmentResource) -> None:
        self._driver_assignment = driver_assignment

        self.assign = async_to_streamed_response_wrapper(
            driver_assignment.assign,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["BatchCreateResponse"]


class BatchCreateResponse(BaseModel):
    msg: Optional[str] = None
    """Displays the error message in case of a failed request or operation.

    Please note that this parameter is not returned in the response in case of a
    successful request.
    """

    status: Optional[str] = None
    """Returns the overall status of the API request.

    Its value will belong to one of `success`, `failed`, and `pending`. It can also
    contain HTTP error codes in case of a failed request or operation.
    """

    track_id: Optional[str] = None
    """Returns the unique ID of the batch processing task.

    Use this ID using the GET request to retrieve the solution once the task
    processing is completed.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.distancematrix import json_retrieve_params
from ...types.distancematrix.json_retrieve_response import JsonRetrieveResponse

__all__ = ["JsonResource", "AsyncJsonResource"]


class JsonResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> JsonResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return JsonResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JsonResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return JsonResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """asfd"""
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/distancematrix/json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def retrieve(
        self,
        *,
        destinations: str,
        key: str,
        origins: str,
        approaches: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        avoid: Literal["toll", "ferry", "highway", "none"] | NotGiven = NOT_GIVEN,
        bearings: str | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        route_failed_prompt: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> JsonRetrieveResponse:
        """
        Nextbillion.ai Distance Matrix API computes distances and ETAs between a set of
        origins and destinations — could be for one-to-many or many-to-many scenarios.
        The API call returns a matrix of ETAs and distances for each origin and
        destination pair. For example, If the set is Origins {A,B} and Destinations
        {C,D,E} we can get the following set of results with distance (meters) and time
        (seconds) for each. The GET method can only handle up to 100 locations (1
        location is either 1 origin or 1 destination).

        Args:
          destinations: "destinations" are the ending coordinates of your route. Ensure that
              "destinations" are routable land locations. Multiple "destinations" should be
              separated by a pipe symbol "|".

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          origins: "origins" are the starting point of your route. Ensure that "origins" are
              routable land locations. Multiple "origins" should be separated by a pipe symbol
              "|".

          approaches: A semicolon-separated list indicating the side of the road from which the route
              will approach "destinations". When set to "unrestricted" a route can arrive at a
              destination from either side of the road. When set to "curb" the route will
              arrive at a destination on the driving side of the region. Please note the
              number of values provided must be equal to the number of "destinations".
              However, you can skip a coordinate and show its position in the list with the
              ";" separator. The values provided for the "approaches" parameter are effective
              for the "destinations" value at the same index. Example: "curb;;curb" will apply
              curbside restriction on the "destinations" points provided at the first and
              third index.

          avoid: Setting this will ensure the route avoids ferries, tolls, highways or nothing.
              Multiple values should be separated by a pipe (|). If "none" is provided along
              with other values, an error is returned as a valid route is not feasible. Please
              note that when this parameter is not provided in the input, ferries are set to
              be avoided by default. When this parameter is provided, only the mentioned
              objects are avoided.

          bearings: Limits the search to segments with given bearing in degrees towards true north
              in clockwise direction. Each "bearing" should be in the format of
              "degree,range", where the "degree" should be a value between \\[[0, 360\\]] and
              "range" should be a value between \\[[0, 180\\]]. Please note that the number of
              "bearings" should be equal to the sum of the number of points in "origins" and
              "destinations". If a route can approach a destination from any direction, the
              bearing for that point can be specified as "0,180".

          mode: Set which driving mode the service should use to determine the "distance" and
              "duration" values. For example, if you use "car", the API will return the
              duration and distance of a route that a car can take. Using "truck" will return
              the same for a route a truck can use, taking into account appropriate truck
              routing restrictions.

              When "mode=truck", following are the default dimensions that are used:

              \\-- truck_height = 214 centimeters

              \\-- truck_width = 183 centimeters

              \\-- truck_length = 519 centimeters

              \\-- truck_weight = 5000 kg

              Please use the Distance Matrix Flexible version if you want to use custom truck
              dimensions.

              Note: Only the "car" profile is enabled by default. Please note that customized
              profiles (including "truck") might not be available for all regions. Please
              contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
              representative or reach out at
              [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
              additional profiles.

          route_failed_prompt: A prompt to modify the response in case no feasible route is available for a
              given pair of origin and destination. When set to "true", a value of "-1" is
              returned for those pairs in which:

              \\-- Either origin or the destination can not be snapped to a nearest road. Please
              note that if all the origins and destinations in a request can't be snapped to
              their nearest roads, a 4xx error is returned instead, as the entire request
              failed.

              \\-- Both origin and destination can be snapped to the nearest road, but the
              service can't find a valid route between them. However, a value of "0" is
              returned if both the origin and destination are snapped to the same location.

              "false" is the default value. In this case, a "0" value is returned for all the
              above cases. A 4xx error is returned, in this case as well, when all origins and
              destinations in the request can't be snapped to their nearest road.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/distancematrix/json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "destinations": destinations,
                        "key": key,
                        "origins": origins,
                        "approaches": approaches,
                        "avoid": avoid,
                        "bearings": bearings,
                        "mode": mode,
                        "route_failed_prompt": route_failed_prompt,
                    },
                    json_retrieve_params.JsonRetrieveParams,
                ),
            ),
            cast_to=JsonRetrieveResponse,
        )


class AsyncJsonResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncJsonResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncJsonResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJsonResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncJsonResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """asfd"""
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/distancematrix/json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def retrieve(
        self,
        *,
        destinations: str,
        key: str,
        origins: str,
        approaches: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        avoid: Literal["toll", "ferry", "highway", "none"] | NotGiven = NOT_GIVEN,
        bearings: str | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        route_failed_prompt: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> JsonRetrieveResponse:
        """
        Nextbillion.ai Distance Matrix API computes distances and ETAs between a set of
        origins and destinations — could be for one-to-many or many-to-many scenarios.
        The API call returns a matrix of ETAs and distances for each origin and
        destination pair. For example, If the set is Origins {A,B} and Destinations
        {C,D,E} we can get the following set of results with distance (meters) and time
        (seconds) for each. The GET method can only handle up to 100 locations (1
        location is either 1 origin or 1 destination).

        Args:
          destinations: "destinations" are the ending coordinates of your route. Ensure that
              "destinations" are routable land locations. Multiple "destinations" should be
              separated by a pipe symbol "|".

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          origins: "origins" are the starting point of your route. Ensure that "origins" are
              routable land locations. Multiple "origins" should be separated by a pipe symbol
              "|".

          approaches: A semicolon-separated list indicating the side of the road from which the route
              will approach "destinations". When set to "unrestricted" a route can arrive at a
              destination from either side of the road. When set to "curb" the route will
              arrive at a destination on the driving side of the region. Please note the
              number of values provided must be equal to the number of "destinations".
              However, you can skip a coordinate and show its position in the list with the
              ";" separator. The values provided for the "approaches" parameter are effective
              for the "destinations" value at the same index. Example: "curb;;curb" will apply
              curbside restriction on the "destinations" points provided at the first and
              third index.

          avoid: Setting this will ensure the route avoids ferries, tolls, highways or nothing.
              Multiple values should be separated by a pipe (|). If "none" is provided along
              with other values, an error is returned as a valid route is not feasible. Please
              note that when this parameter is not provided in the input, ferries are set to
              be avoided by default. When this parameter is provided, only the mentioned
              objects are avoided.

          bearings: Limits the search to segments with given bearing in degrees towards true north
              in clockwise direction. Each "bearing" should be in the format of
              "degree,range", where the "degree" should be a value between \\[[0, 360\\]] and
              "range" should be a value between \\[[0, 180\\]]. Please note that the number of
              "bearings" should be equal to the sum of the number of points in "origins" and
              "destinations". If a route can approach a destination from any direction, the
              bearing for that point can be specified as "0,180".

          mode: Set which driving mode the service should use to determine the "distance" and
              "duration" values. For example, if you use "car", the API will return the
              duration and distance of a route that a car can take. Using "truck" will return
              the same for a route a truck can use, taking into account appropriate truck
              routing restrictions.

              When "mode=truck", following are the default dimensions that are used:

              \\-- truck_height = 214 centimeters

              \\-- truck_width = 183 centimeters

              \\-- truck_length = 519 centimeters

              \\-- truck_weight = 5000 kg

              Please use the Distance Matrix Flexible version if you want to use custom truck
              dimensions.

              Note: Only the "car" profile is enabled by default. Please note that customized
              profiles (including "truck") might not be available for all regions. Please
              contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
              representative or reach out at
              [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
              additional profiles.

          route_failed_prompt: A prompt to modify the response in case no feasible route is available for a
              given pair of origin and destination. When set to "true", a value of "-1" is
              returned for those pairs in which:

              \\-- Either origin or the destination can not be snapped to a nearest road. Please
              note that if all the origins and destinations in a request can't be snapped to
              their nearest roads, a 4xx error is returned instead, as the entire request
              failed.

              \\-- Both origin and destination can be snapped to the nearest road, but the
              service can't find a valid route between them. However, a value of "0" is
              returned if both the origin and destination are snapped to the same location.

              "false" is the default value. In this case, a "0" value is returned for all the
              above cases. A 4xx error is returned, in this case as well, when all origins and
              destinations in the request can't be snapped to their nearest road.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/distancematrix/json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "destinations": destinations,
                        "key": key,
                        "origins": origins,
                        "approaches": approaches,
                        "avoid": avoid,
                        "bearings": bearings,
                        "mode": mode,
                        "route_failed_prompt": route_failed_prompt,
                    },
                    json_retrieve_params.JsonRetrieveParams,
                ),
            ),
            cast_to=JsonRetrieveResponse,
        )


class JsonResourceWithRawResponse:
    def __init__(self, json: JsonResource) -> None:
        self._json = json

        self.create = to_raw_response_wrapper(
            json.create,
        )
        self.retrieve = to_raw_response_wrapper(
            json.retrieve,
        )


class AsyncJsonResourceWithRawResponse:
    def __init__(self, json: AsyncJsonResource) -> None:
        self._json = json

        self.create = async_to_raw_response_wrapper(
            json.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            json.retrieve,
        )


class JsonResourceWithStreamingResponse:
    def __init__(self, json: JsonResource) -> None:
        self._json = json

        self.create = to_streamed_response_wrapper(
            json.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            json.retrieve,
        )


class AsyncJsonResourceWithStreamingResponse:
    def __init__(self, json: AsyncJsonResource) -> None:
        self._json = json

        self.create = async_to_streamed_response_wrapper(
            json.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            json.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .batch import (
    BatchResource,
    AsyncBatchResource,
    BatchResourceWithRawResponse,
    AsyncBatchResourceWithRawResponse,
    BatchResourceWithStreamingResponse,
    AsyncBatchResourceWithStreamingResponse,
)
from .console import (
    ConsoleResource,
    AsyncConsoleResource,
    ConsoleResourceWithRawResponse,
    AsyncConsoleResourceWithRawResponse,
    ConsoleResourceWithStreamingResponse,
    AsyncConsoleResourceWithStreamingResponse,
)
from .geofence import (
    GeofenceResource,
    AsyncGeofenceResource,
    GeofenceResourceWithRawResponse,
    AsyncGeofenceResourceWithRawResponse,
    GeofenceResourceWithStreamingResponse,
    AsyncGeofenceResourceWithStreamingResponse,
)

__all__ = [
    "ConsoleResource",
    "AsyncConsoleResource",
    "ConsoleResourceWithRawResponse",
    "AsyncConsoleResourceWithRawResponse",
    "ConsoleResourceWithStreamingResponse",
    "AsyncConsoleResourceWithStreamingResponse",
    "BatchResource",
    "AsyncBatchResource",
    "BatchResourceWithRawResponse",
    "AsyncBatchResourceWithRawResponse",
    "BatchResourceWithStreamingResponse",
    "AsyncBatchResourceWithStreamingResponse",
    "GeofenceResource",
    "AsyncGeofenceResource",
    "GeofenceResourceWithRawResponse",
    "AsyncGeofenceResourceWithRawResponse",
    "GeofenceResourceWithStreamingResponse",
    "AsyncGeofenceResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AssetTrackParams", "Locations", "LocationsLocation"]


class AssetTrackParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    device_id: Required[str]
    """ID of the device used to upload the tracking information of the `asset`.

    Please note that the `device_id` used here must already be linked to the
    `asset`. Use the _Bind Device to Asset_ method to link a device with your
    `asset`.
    """

    locations: Required[Locations]
    """An array of objects to collect the location tracking information for an `asset`.

    Each object must correspond to details of only one location.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""


class LocationsLocation(TypedDict, total=False):
    lat: Required[float]
    """Latitude of the tracked location of the `asset`."""

    lon: Required[float]
    """Longitude of the tracked location of the `asset`."""


class Locations(TypedDict, total=False):
    location: Required[LocationsLocation]
    """An object to collect the coordinate details of the tracked location.

    Please note this field is mandatory when uploading locations for an asset.
    """

    timestamp: Required[int]
    """
    Use this parameter to provide the time, expressed as UNIX epoch timestamp in
    milliseconds, when the location was tracked. Please note this field is mandatory
    when uploading locations for an asset.
    """

    accuracy: float
    """
    Use this parameter to provide the accuracy of the GPS information at the tracked
    location. It is the estimated horizontal accuracy radius, in meters.
    """

    altitude: float
    """
    Use this parameter to provide the altitude, in meters, of the `asset` at the
    tracked location.
    """

    battery_level: int
    """
    Use this parameter to provide the battery level of the GPS device, as a
    percentage, when the location is tracked. It should have a minimum value of 0
    and a maximum value of 100.
    """

    bearing: float
    """
    Use this parameter to provide the heading of the `asset`, in radians, calculated
    from true north in clockwise direction. This should always be in the range of
    [0, 360).
    """

    meta_data: object
    """Use this object to add any custom data about the location that is being
    uploaded.

    Recommended to use the `key`:`value` format for adding the desired information.

    Please note that the maximum size of `meta_data` object should not exceed 65Kb.
    """

    speed: float
    """
    Use this parameter to provide the speed of the `asset`, in meters per second, at
    the tracked location.
    """

    tracking_mode: str
    """NB tracking mode."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from .metadata_param import MetadataParam

__all__ = ["MonitorCreateParams", "GeofenceConfig", "IdleConfig", "MatchFilter", "SpeedingConfig"]


class MonitorCreateParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    tags: Required[List[str]]
    """Use this parameter to add `tags` to the `monitor`.

    `tags` can be used for filtering monitors in the _Get Monitor List_ operation.
    They can also be used for easy identification of monitors.

    Please note that valid `tags` are strings, consisting of alphanumeric characters
    (A-Z, a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.
    """

    type: Required[Literal["`enter`", "`exit`", "`enter_and_exit`", "`speeding`", "`idle`"]]
    """Specify the type of activity the `monitor` would detect.

    The `monitor` will be able to detect the specified `type` of activity and create
    events for eligible `asset`. A `monitor` can detect following types of asset
    activity:

    - `enter`: The `monitor` will create an event when a linked `asset` enters into
      the specified geofence.

    - `exit`: The `monitor` will create an event when a linked `asset` exits the
      specified geofence.

    - `enter_and_exit`: The `monitor` will create an event when a linked `asset`
      either enters or exits the specified geofence.

    - `speeding`: The `monitor` will create an event when a linked `asset` exceeds a
      given speed limit.

    - `idle`: The `monitor` will create an event when a linked `asset` exhibits idle
      activity.

    Please note that `assets` and geofences can be linked to a `monitor` using the
    `match_filter` and `geofence_config` attributes respectively.
    """

    cluster: Literal["america"]
    """the cluster of the region you want to use"""

    custom_id: str
    """Set a unique ID for the new `monitor`.

    If not provided, an ID will be automatically generated in UUID format. A valid
    `custom_id` can contain letters, numbers, "-", & "\\__" only.

    Please note that the ID of an `monitor` can not be changed once it is created.
    """

    description: str
    """Add a description for your `monitor` using this parameter."""

    geofence_config: GeofenceConfig
    """Geofences are geographic boundaries surrounding an area of interest.

    `geofence_config` is used to specify the geofences for creating `enter` or
    `exit` type of events based on the asset's location. When an asset associated
    with the `monitor` enters the given geofence, an `enter` type event is created,
    whereas when the asset moves out of the geofence an `exit` type event is
    created.

    Please note that this object is mandatory when the monitor `type` belongs to one
    of `enter`, `exit` or `enter_and_exit`.
    """

    geofence_ids: List[str]
    """\\**\\**Deprecated.

    Please use the `geofence_config` to specify the geofence_ids for this
    monitor.\\**\\**

    An array of strings to collect the geofence IDs that should be linked to the
    `monitor`. Geofences are geographic boundaries that can be used to trigger
    events based on an asset's location.
    """

    idle_config: IdleConfig
    """`idle_config` is used to set up constraints for creating idle events.

    When an asset associated with the `monitor` has not moved a given distance
    within a given time, the Live Tracking API can create events to denote such
    instances. Please note that this object is mandatory when the monitor `type` is
    `idle`.

    Let's look at the properties of this object.
    """

    match_filter: MatchFilter
    """
    This object is used to identify the asset(s) on which the `monitor` would be
    applied.
    """

    meta_data: MetadataParam
    """Any valid json object data.

    Can be used to save customized data. Max size is 65kb.
    """

    name: str
    """Name of the `monitor`.

    Use this field to assign a meaningful, custom name to the `monitor` being
    created.
    """

    speeding_config: SpeedingConfig
    """`speeding_config` is used to set up constraints for creating over-speed events.

    When an `asset` associated with a `monitor` is traveling at a speed above the
    given limits, the Live Tracking API can create events to denote such instances.
    There is also an option to set up a tolerance before creating an event. Please
    note that this object is mandatory when `type=speeding`.

    Let's look at the properties of this object.
    """


class GeofenceConfig(TypedDict, total=False):
    geofence_ids: Required[List[str]]
    """
    An array of strings to collect the geofence IDs that should be linked to the
    `monitor`. Please note `geofence_ids` are mandatory when using the
    `geofence_config` attribute.
    """


class IdleConfig(TypedDict, total=False):
    distance_tolerance: Required[float]
    """
    Use this parameter to configure a distance threshold that will be used to
    determine if the asset was idle or not. If the asset moves by a distance less
    than the value of this parameter within a certain time period, the `monitor`
    would create an idle event against the asset. The `distance_tolerance` should be
    provided in meters.

    Users can set an appropriate value for this parameter, along with appropriate
    `time_tolerance` value, to avoid triggering idle events when the asset is
    crossing a busy intersection or waiting at the traffic lights.
    """

    time_tolerance: int
    """
    Use this parameter to configure a time duration for which the `monitor` would
    track the distance covered by an asset before triggering an idle event. The
    `time_tolerance` should be provided in milliseconds.

    If the distance covered by the asset during a `time_tolerance` is less than that
    specified in `distance_tolerance` the asset will be assumed to be idle.

    Please observe that this attribute along with `distance_tolerance` parameter can
    be used to control the "sensitivity" of the `monitor` with respect to idle
    alerts. If the `distance_tolerance` is set a high value, then setting
    `time_tolerance` to a low value may result in a situation where asset is always
    judged as idle. On the contrary, it might never be judged as idle if
    `distance_tolerance` is set to a low value but `time_tolerance` is set to a high
    value.

    It is recommended to use these properties with appropriate values to trigger
    genuine idle events. The appropriate values might depend on the traffic
    conditions, nature of operations that the asset is involved in, type of asset
    and other factors.
    """


class MatchFilter(TypedDict, total=False):
    include_all_of_attributes: object
    """A string type dictionary object to specify the `attributes`.

    Only the assets having all of the `attributes` added to this parameter will be
    linked to this `monitor`. Once an `asset` is linked to a `monitor`, the
    `monitor` will be able to create events for that `asset` whenever an activity
    specified in `type` is detected. Multiple attributes should be separated by a
    comma `,`.

    Please note that this parameter can not be used in conjunction with
    `include_any_of_attributes`. Also, the maximum number of `key`:`value` pairs
    that this parameter can take is 100 and the overall size of the `match_filter`
    object should not exceed 65kb.
    """

    include_any_of_attributes: object
    """A string type dictionary object to specify the `attributes`.

    The assets having at least one of the `attributes` added to this parameter will
    be linked to this `monitor`. Once an `asset` is linked to a `monitor`, the
    `monitor` will be able to create events for that `asset` whenever an activity
    specified in `type` is detected. Multiple attributes should be separated by a
    comma `,`.

    Please note that this parameter can not be used in conjunction with
    `include_all_of_attributes`. Also, the maximum number of `key`:`value` pairs
    that this parameter can take is 100 and the overall size of the `match_filter`
    object should not exceed 65kb.
    """


class SpeedingConfig(TypedDict, total=False):
    customer_speed_limit: int
    """
    Use this parameter to establish the speed limit that will allow the `monitor` to
    create events, depending on the `time_tolerance` value, when an asset's tracked
    speed exceeds it. The speed limit should be specified in meters per second.

    Please note that `customer_speed_limit` is mandatory when
    `use_admin_speed_limit` is false. However, when `use_admin_speed_limit` is true,
    `customer_speed_limit` is ineffective.
    """

    time_tolerance: int
    """Use this parameter to configure a time tolerance before triggering an event.

    Adding a tolerance would make the Tracking service wait for the specified time
    before triggering the event. Consequently, an event is triggered only when the
    time for which the `asset` has been over-speeding continuously, exceeds the
    configured tolerance time. The unit for this parameter is milliseconds.

    It can be seen that this attribute is used to control the "sensitivity" of the
    `monitor` with respect to speed alerts. Higher the value of `time_tolerance` the
    less sensitive the `monitor` would be to instances of over-speeding. Conversely,
    if 'time_tolerance' is set to 0, the `monitor` will be extremely sensitive and
    will create an event as soon as tracking information with a speed value greater
    than the specified limit is received.
    """

    use_admin_speed_limit: bool
    """
    A boolean attribute to indicate which speed limit values should be used by the
    `monitor`. When `use_admin_speed_limit` is true, the administrative speed limit
    of the road on which the asset is located, will be used to generate events when
    the asset’s tracked speed exceeds it. Whereas, when `use_admin_speed_limit` is
    false, the `customer_speed_limit` specified will be used to generate events when
    the asset's tracked speed exceeds it.

    Please note that if `use_admin_speed_limit` is false, `customer_speed_limit` is
    mandatory, however, when `use_admin_speed_limit` is true then
    `customer_speed_limit` is ineffective.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["TrackLocation", "Location"]


class Location(BaseModel):
    lat: Optional[float] = None
    """Latitude of the tracked location of the `asset`."""

    lon: Optional[float] = None
    """Longitude of the tracked location of the `asset`."""


class TrackLocation(BaseModel):
    accuracy: Optional[float] = None
    """
    If available, this property returns the accuracy of the GPS information received
    at the tracked location. It is represented as an estimated horizontal accuracy
    radius, in meters, at the 68th percentile confidence level.
    """

    altitude: Optional[float] = None
    """
    If available in the GPS information, this property returns the altitude of the
    `asset` at the tracked location. It is represented as height, in meters, above
    the WGS84 reference ellipsoid.
    """

    battery_level: Optional[int] = None
    """
    Returns the battery level of the GPS device, as a percentage, when the location
    was tracked. It has a minimum value of 0 and a maximum value of 100.
    """

    bearing: Optional[float] = None
    """
    If available in the GPS information, this property returns the heading of the
    `asset` calculated from true north in clockwise direction at the tracked
    location. Please note that the bearing is not affected by the device
    orientation.

    The bearing will always be in the range of [0, 360).
    """

    location: Optional[Location] = None
    """An object with the coordinates of the last tracked location."""

    meta_data: Optional[object] = None
    """
    Specifies the custom data about the location that was added when the location
    was uploaded.
    """

    speed: Optional[float] = None
    """
    If available in the GPS information, this property returns the speed of the
    `asset`, in meters per second, at the tracked location.
    """

    timestamp: Optional[int] = None
    """
    A UNIX epoch timestamp in milliseconds, representing the time at which the
    location was tracked.
    """

    tracking_mode: Optional[str] = None
    """Internal parameter for tracking mode."""
